/*
 * Check if the PID from the command-line belongs to a mgetty- or vgetty-process.
 * If it does, send it a 'kill -10' to pick up the phone.
 * This program must be run SUID.
 * Usage:  do_ata mgetty-PID
 *
 * Copyleft  (C) 1995 Tim Bandelow <bandelow@busch.bauwesen.uni-dortmund.de>
 */

#include <fcntl.h>
#include <string.h>
#include <signal.h>

#ifdef __STDC__
#define _P1(x,a1)	(a1)
#else
#define _P1(x,a1)	x a1;
#endif

/* get the base file name of a file path
 * ( shamelessly stolen from goodies.c from the mgetty kit )
 */

char * get_basename _P1( (s), char * s )
{
char * p;

    if ( s == NULL ) return NULL;

    p = strrchr( s, '/' );

    return ( p == NULL ) ? s: p+1;
}

/* return process name + arguments for process "PID"
 * use /proc filesystem on Linux and SVR4
 * if no information is available, return NULL
 * ( shamelessly stolen from goodies.c from the mgetty kit )
 */

char * get_ps_args _P1 ((pid), int pid )
{
#ifdef SVR4
    char *pscomm = NULL;
# ifdef PIOCPSINFO
    int procfd;
    char procfname[12];
    static prpsinfo_t psi;

    sprintf (procfname, "/proc/%05d", pid);
    procfd = open (procfname, O_RDONLY);
    if ( procfd >= 0 ) {
		if (ioctl (procfd, PIOCPSINFO, &psi) != -1) {
		    psi.pr_psargs[PRARGSZ-1] = '\0';
		    pscomm = psi.pr_psargs;
		}
		close(procfd);
    }
# endif /* PIOCPSINFO */
    return pscomm;
#endif /* SVR4 */

#ifdef linux

    char procfn[30];
    int procfd;
    int i,l;
    static char psinfo[60];	/* 60 is considered long enough */

    sprintf( procfn, "/proc/%d/cmdline", pid );
    procfd = open( procfn, O_RDONLY );
    if ( procfd < 0 ) return NULL;
    l = read( procfd, psinfo, sizeof( psinfo ) -1 );
    if ( l < 0 ) {
		close( procfd );
		return NULL;
    }
    close( procfd );
    psinfo[l] = 0;

    /* arguments separated by \0, replace with space */
    for ( i=0; i<l; i++ )
	if ( psinfo[i] == 0 ) psinfo[i] = ' ';
    return psinfo;
#endif /* linux */

#if !defined(SVR4) && !defined(linux)
    return NULL;
#endif
}


int main(int argc, char* argv[])
{
    int pid, i;
    char ps_name[60], *ps_info, *ps_base, command[25];

    if (argc != 2) return 1;
    sscanf(argv[1], "%d", &pid);
    ps_info = get_ps_args(pid);
    if (ps_info == NULL) return 1;
    for (i=0 ; (i<strlen(ps_info)) && (ps_info[i]!=' ') ; i++) ps_name[i]=ps_info[i];
    ps_name[i] = '\0';
    ps_base = get_basename(ps_name);
    if (!strcmp(ps_base, "mgetty") || !strcmp(ps_base, "vgetty")) return(kill(pid, 10));
    return 1;
}
