# ------------------------------------------------------------------------------
# DTMF-Commands
# ------------------------------------------------------------------------------
# DTMF-Data-File-Format (the Index file)
# DATAFORMAT : CODE1
#              CODE1    FileName1
#              CODE1    FileName2
#              CODE2
#              CODE2    FileName2
# CODE is exact 8 Bytes long (filled up with Spaces!) + one Space as delimiter
#
# this is internal coded as follows
#
# 1. only CodeEntry selected (watch the special display)
#         HINT != "" FILE == ""
# 2. only FileEntry selected
#         HINT == "" FILE != ""
# 3. Code&FileEntry selected
#         HINT != "" FILE != ""
# ------------------------------------------------------------------------------

# Update the directory-listbox for DTMF

proc do_DTMF_update {} {
  global curr_dir curr_file index_db hint_list new_list show_multi

  if ![file readable $curr_dir] {log_entry "Can't read $curr_dir" ; return 0}
  update_start
  read_DTMF
  set last_hint ""
  set count_hint 0
  foreach i $index_db {
    set hint "[lindex $i 0]        "
    set hint "[string range $hint 0 7]"
    if !$show_multi {
      if {$new_list != "TRUE" && [lsearch $hint_list $hint] >= 0} {
#.......Code Line exists ... now enter the Files
        set file_name [lindex $i 1]
        set file_text "          $file_name"
        insert_list "$file_name" "$file_text" "$hint"
      } else {
#.......code is new ... so create a CodeLine only
#.......detected by filename == ""
        set file_name ""
        set file_text [string range "[string trim $hint]........" 0 7]:
        insert_list "$file_name" "$file_text" "$hint"
#.......and the normal HINT Filename Line with unvisable Codes
        set file_name [lindex $i 1]
        if { $file_name != "" } {
          set file_text "          $file_name"
          insert_list "$file_name" "$file_text" "$hint"
        }
      }
    } else {
#.....show_multi
      if { "$last_hint" == "" } { set last_hint "$hint" }
      if { "$last_hint" == "$hint" } {
        if { [lindex $i 1] != "" } { incr count_hint }
      } else {
        set file_name ""
        set file_text \
          [string range "[string trim $last_hint]........" 0 7]:\ ($count_hint)
        insert_list "$file_name" "$file_text" "$last_hint"
        if { [lindex $i 1] != "" } { set count_hint 1 } else { set count_hint 0 }
      }
      set last_hint "$hint"
    }      
  }
#.last show_multi
  if $show_multi {
    set file_name ""
    set file_text [string range "[string trim $last_hint]........" 0 7]:\ ($count_hint)
    insert_list "$file_name" "$file_text" "$last_hint"
  } 
  insert_list "" "messages:" "" 
#.Append All Filenames detected by hint == ""
  set files [lsort [glob -nocomplain $curr_dir/$curr_file]]
  set hint ""
  foreach f $files {
    if ![file isdirectory $f] {
      set file_name "[file tail $f]"
#.....don't show file 'Index' 
      if {$file_name != "Index"} {
        set file_text "          $file_name"
        insert_list "$file_name" "$file_text" "$hint"
      }
    } 
  }
  update_end
}


# All FileOnly entries will be activatet for all selected CodeOnly Lines
# selected Code&FileEntries will be deactivated
# if no CodeOnly but FileOnly entries are selected an private DTMF code is requested
# and added

proc do_DTMF_activate {} {
  global file_list hint_list index_db DTMF_file curr_mode n2

  set items [listbox_selection]
  if {$items == ""} {return 0}
  if ![file writable $DTMF_file] \
     {log_entry "Activate: can't write to $DTMF_file" ; return 0}
  set active [listbox_active]
  foreach line $items {
    set n [lindex $file_list $line]
#...look for FileOnly entry
    if {[lindex $hint_list $line] == "" && [lindex $file_list $line] != "" } {
      log_entry "$n: Activating file"
      set code 0
      foreach line2 $items {
        set n2 [lindex $hint_list $line2]
#.......look for CodeOnly entry
        if {[lindex $file_list $line2] == "" && [lindex $hint_list $line2] != "" } {
          log_entry "for Code $n2:"
          set code 1
#.........and activate them together
          lappend index_db "$n2 $n"
        }
      }
#.....if no CodeOnly entry was found request code input
      if { $code == 0 } {
        set n2 ""
        if { ![interact entry Activate n2] } {
#.........delete all non_numbers:
          regsub -all "\[^0-9\]" "$n2" "" n2
          if {"$n2" != "" } {
            set n2 [string range "$n2        " 0 7]
            lappend index_db "$n2 $n"
          }
        }
      }
    } else {
#...look for CodeFile entry and deactivate them
      if {[lindex $hint_list $line] != "" && [lindex $file_list $line] != "" } {
        DTMF_delete "[lindex $hint_list $line] [lindex $file_list $line]"
      }
    }
  }
  write_DTMF
  do_mmail_mod $curr_mode
}


proc DTMF_delete {entry} {
  global index_db

  set i [lsearch -exact "$index_db" "$entry"]
  if {$i < 0} {return 0}
  if {$i > 0} {
    if {$i == [expr [llength $index_db] - 1]} {
      set index_db [lrange $index_db 0 [expr $i - 1]]
    } else {
      set index_db [concat [lrange $index_db 0 [expr $i - 1]] \
                           [lrange $index_db [expr $i + 1] end]]
    }
  } else {
    if {[llength $index_db] == 1} {set index_db ""} \
    else {set index_db [lrange $index_db [expr $i + 1] end]}
  }
}


# Read the DTMF-file

proc read_DTMF {} {
  global curr_dir index_db text DTMF_file

  set index_db {}
  set error 0
  if {![catch {open "$DTMF_file" r} file_id]} {
    while {![eof $file_id]} {
      gets $file_id line
      if {[file exists $curr_dir/[lindex $line 1]]} { 
        if {$line != ""} {
          lappend index_db "$line"
        }
      } else {set error 1}
    }
    catch {close $file_id}
  }
#.here a correction should be done!
#.FIRST add a Line for each Code without a Filename!
#.SECOND remove all equal lines !
#.this is braindemaged but it works ;-)
#.and all other function trust this routine
  foreach line "$index_db" \
    {lappend index_db [string range "[lindex $line 0]        " 0 7]\ }
  set correction 1
  while { $correction == 1 } {
    set correction 0
    foreach line "$index_db" {
      set first [lsearch -exact "$index_db" "$line"]
      incr first
      if { [llength "$index_db"] > $first } {
        if { [lsearch -exact [lrange "$index_db" $first end] "$line"] > -1 } {
          DTMF_delete "$line"
          set correction 1
          if { [lindex "$line" 1] != "" } {set error 1} 
          break
        }
      }
    }
  }
  if $error {
    log_entry "WARNING: Corrected errors in DTMF_Index"
  }
  set index_db [lsort "$index_db"]
  write_DTMF
}


# Write the DTMF-file

proc write_DTMF {} {
  global index_db DTMF_file

  if {[catch {open $DTMF_file w} file_id]} {
    log_entry "Can't write to $DTMF_file"
  } else {
    foreach entry "$index_db" {
      if {"$entry" != ""} {puts $file_id "$entry"}
    }
    catch {close $file_id}
  }
}


# Rename the DTMF entries.
# Files and/or Codes can be renamed
# and a correction of the Index file is done.
# Some selections are ignored since there is no need for an action

proc do_DTMF_rename {} {
  global file_list hint_list index_db DTMF_file curr_mode newname curr_dir text

  set items [listbox_selection]
  if {$items == ""} {return 0}
  if ![file writable $DTMF_file] \
     {log_entry "Rename: can't write to $DTMF_file" ; return 0}
  if ![file writable $curr_dir] \
     {log_entry "Rename: can't write to $curr_dir" ; return 0} 
  set active [listbox_active]
  foreach line $items {
#...look for FileOnly entry
    if {[lindex $hint_list $line] == "" && [lindex $file_list $line] != "" } {
      set newname [lindex $file_list $line]
      set oldname "$newname"
      if {![interact entry Rename newname] && "$newname" != ""} {
        if {"$newname" == "Index"} {
          log_entry "Rename: Can't use reserved name '$newname'"
        } else {
#.........replace spaces with underscores:
          regsub -all " " "$newname" "_" newname
          if {"$oldname" != "$newname"} {
#...........Check if there is already a file with this name
            if [file exists "$curr_dir/$newname"] {
              set text "Overwrite existing file $newname ?"
              set do_it ![interact label OK text]
              if $do_it {set overwrite 1}
            } else {set do_it 1 ; set overwrite 0}
            if $do_it {
              log_entry "$oldname: Renaming file"
              set command "mv -f \"$curr_dir/$oldname\" \"$curr_dir/$newname\""
              if [catch "eval exec $command"] {
                log_entry "$oldname: Renaming file -- failed"
              } else {
#...............renaming done
#...............now repair the Index File !
#...............enter a CodeLine for all existing Codes w/ the old FileName
#...............with the newFileName (the old_ones will be deleted by DTMF_read
                foreach line2 "$index_db" {
                  if {"[lindex $line2 1]" == "$oldname"} {
                    lappend index_db \
                        [string range "[lindex $line2 0]        " 0 7]\ $newname
                  }
                }                    
              }
            }
          }
        }
      }
    } else {
#.....look for CodeOnly entry
      if {[lindex $file_list $line] == "" && [lindex $hint_list $line] != "" } {
        set newname [string trim [lindex $hint_list $line]]
        set oldname "$newname"
        if {![interact entry Rename newname]}  {
#.........delete all non_numbers:
          regsub -all "\[^0-9\]" "$newname" "" newname
          if { "$newname" != ""} {
            log_entry "$oldname: Renaming code"
            foreach line2 "$index_db" {
              if {"[string trim [lindex $line2 0]]" == "$oldname"} {
                DTMF_delete "$line2"
                lappend index_db [string range "$newname        " 0 7]\ [lindex $line2 1]
              }
            }
          }
        }
      }
    }
  }
  write_DTMF
  do_mmail_mod $curr_mode
}


# Delete DTMF entries.
# Files and/or Codes can be deleted
# and a correction of the Index file is done.
# Some selections are ignored since there is no need for an action

proc do_DTMF_remove {} {
  global file_list hint_list index_db DTMF_file curr_mode newname curr_dir text

  set items [listbox_selection]
  if {$items == ""} {return 0}
  if ![file writable $DTMF_file] \
     {log_entry "Rename: can't write to $DTMF_file" ; return 0}
  if ![file writable $curr_dir] \
     {log_entry "Rename: can't write to $curr_dir" ; return 0} 
  set active [listbox_active]
  foreach line $items {
#...look for FileOnly entry
    if {[lindex $hint_list $line] == "" && [lindex $file_list $line] != "" } {
      set text "Delete selected file: [lindex $file_list $line] ?"
      set ret [interact label Delete text]
      if !$ret {
        log_entry "[lindex $file_list $line]: Removing file"
        catch "eval exec rm -f $curr_dir/[lindex $file_list $line] >/dev/null"
      }
    } else {
#.....look for CodeOnly entry
      if {[lindex $file_list $line] == "" && [lindex $hint_list $line] != "" } {
        set text "Delete selected code: [string trim [lindex $hint_list $line]] ?"
        set ret [interact label Delete text]
        if !$ret {
          log_entry "[lindex $hint_list $line]: Removing code"
          set correction 1
          while { $correction == 1 } {
            set correction 0
            foreach line2 "$index_db" {
              if { "[string trim [lindex $hint_list $line]]" == \
                   "[string trim [lindex $line2 0]]" } {
                DTMF_delete "$line2"
                set correction 1
                break
              }
            }
          }
        }
      }
    }
  }
  write_DTMF
  do_mmail_mod $curr_mode
}

