# ------------------------------------------------------------------------------
# Exec commands (loosely based on xtem_texmenu)
# ------------------------------------------------------------------------------

# Check whether the specified command is executable

proc which {command} {
  global env
  if [file executable $command] {return 1}
  foreach dir [split $env(PATH) :] {
    set path ${dir}/${command}
    if {[file isfile $path]&&[file executable $path]} {return 1}
  }
  return 0
}


# Executed when the child writes to stdout or stderr (Tcl7.5+)

proc cmd_out {} {
  global f_mkCmd p_mkCmd end_mkCmd first_line new_logline

  if {[catch {eof $f_mkCmd}]} {
    set p_mkCmd 1
    set end_mkCmd "end"
    return
  }
  if {[eof $f_mkCmd]} {
    set p_mkCmd 1
    set end_mkCmd "end"
    catch "close $f_mkCmd"
    return
  }
  set c [read $f_mkCmd]
  if {$c==""} return
  if $first_line {set new_logline 1}
  set first_line 0
  log_event $c
}


# Executed when the child writes to stdout (Tcl7.4/TclX)

proc cmd_stdout {} {
  global o_cmd p_cmd new_logline first_stdout_line first_stderr_line
  set erg [catch "read $o_cmd" c]
  if  {($p_cmd==1)&&(($c=="")||($erg))} then {catch "close $o_cmd"} \
  else {
    if {$c==""} return
    if $first_stdout_line {set new_logline 1}
    set first_stdout_line 0
    set first_stderr_line 1
    log_event $c
  }
}


# Executed when the child writes to stderr (Tcl7.4/TclX)

proc cmd_stderr {} {
  global p_cmd e_cmd new_logline first_stdout_line first_stderr_line
  set erg [catch "read $e_cmd" c]
  if {($p_cmd==1)&&(($c=="")||($erg))} then {catch "close $e_cmd"} \
  else {
    if {$c==""} return
    if $first_stderr_line {set new_logline 1}
    set first_stderr_line 0
    set first_stdout_line 1
    log_event $c
  }
}


# Executed when the child dies (SIGCHLD) (Tcl7.4/TclX)

proc cmd_fin {} {
  global p_cmd i_cmd end_mkCmd
  set res [catch "wait -nohang $p_cmd" message] 
  if {($res==0)&&($message!="")} {
    set p_cmd 1
    cmd_stdout
    cmd_stderr
    catch "close $i_cmd"
    set end_mkCmd "end"
    signal ignore SIGCHLD
    update
  }
}


# Start ONE external command

proc exec_cmd {cmd args {pid_var "cmd_pid"} {wait ""}} {
  global $pid_var argv0 tcl_version end_mkCmd

  if ![which $cmd] {
    log_entry "ERROR: command '$cmd' not found"
    return 0
  }
  flush stdout
  flush stderr
  set end_mkCmd "start"

#.catch command output to display it into the logbox
  if {$tcl_version >= 7.5} {
    global f_mkCmd p_mkCmd first_line

    set first_line 1
    set f_mkCmd [open "| $cmd $args |& cat" RDWR]
    fileevent $f_mkCmd readable "cmd_out"
    fconfigure $f_mkCmd -blocking 0 -buffering none
    set p_mkCmd [pid $f_mkCmd]
    set $pid_var [lindex $p_mkCmd 0]
  } else {
    global i_cmd o_cmd e_cmd p_cmd first_stdout_line first_stderr_line

    signal trap SIGCHLD {cmd_fin}
    set first_stdout_line 1
    set first_stderr_line 1
    pipe ir iw; pipe or ow; pipe er ew
    fileevent $or readable "cmd_stdout"
    fileevent $er readable "cmd_stderr"
    fcntl $or NONBLOCK 1
    fcntl $er NONBLOCK 1
    fcntl $or NOBUF 1 
    fcntl $er NOBUF 1
    if {[set $pid_var [fork]]==0} {
  #...execute command with redirected output
      dup   $ir stdin;  dup   $ow stdout;  dup   $ew stderr
      close $ir;        close $ow;         close $ew
      close $iw;        close $or;         close $er
      execl $cmd $args
  #...just in case it fails (normally shouldn't return)
      kill [id process]
    }
    close $ir;  close $ow;  close $ew
    set i_cmd $iw;  set o_cmd $or;
    set e_cmd $er;  eval set p_cmd $$pid_var
  }

#.wait for command termination if requested
  if {$wait != ""} {tkwait variable end_mkCmd}

#.restore original default bindings
  if [string compare [file tail $argv0] "PalFax"] {
    std_keys
    act_keys 1
  }
  return 1
}

