# ------------------------------------------------------------------------------
# Fax commands
# ------------------------------------------------------------------------------

# View or Print marked faxes in fine or normal resolution

proc do_fax {mode} {
  global do_View_n_command do_View_f_command
  global do_Print_n_command do_Print_f_command
  global curr_dir file_list hint_list tmp_file show_multi

  set items [listbox_selection]
  if {$items == ""} {return 0}
  foreach i $items {
    set n [lindex $file_list $i]
    set files [lsort [glob -nocomplain "$curr_dir/$n"]]
    set all_files ""
    foreach f $files {
      set hint [lindex $hint_list $i]
      if {[file readable $f] && "$hint" != ""} {
        if {$mode == "View" && $show_multi} {
#.........multipage-view: just collect the filenames
          if {"$all_files" == ""} {set all_files $f} \
          else {append all_files " $f"}
        } else {
#.........print / singlepage view
          log_entry "[file tail $f]: ${mode}ing file"
          set command "do_${mode}_${hint}_command 2> $tmp_file &"
          if [catch "eval exec $$command"] {
            log_entry "[file tail $f]: ${mode}ing file -- failed"
          }
#.........look for error-messages
          after 500
          if {[file exists $tmp_file] && [file size $tmp_file]} {
            log_entry [read_file "" $tmp_file]
          }
        }
      } else {log_entry "Can't read [file tail $f]"}
    }
#...multipage-view
    if {$mode == "View" && $show_multi && "$all_files" != ""} {
      set f [eval list $all_files]
      if {[llength $all_files] == 1} {
        set name [string range $f [expr [string length $curr_dir] + 1] end]
        set log_text "$name: Viewing file"
      } else {
        set log_text "$n: Viewing files"
      }
      log_entry $log_text
      set command "do_View_${hint}_command 2> $tmp_file &"
#.....thats TCL :-) 
      if [catch "eval eval exec $$command"] {
        log_entry "$log_text -- failed"
      }
#.....look for error-messages
      after 500
      if {[file exists $tmp_file] && [file size $tmp_file]} {
        log_entry [read_file "" $tmp_file]
      }
    }
  }
  catch "eval exec rm -f $tmp_file"
}


# Crop unneeded lines from the end of a fax page

proc do_crop {} {
  global curr_mode curr_dir file_list text tmp_file do_Crop_command

  set items [listbox_selection]
  set active [listbox_active]
  if {$items == ""} {return 0}
  if ![file writable $curr_dir] \
     {log_entry "$curr_dir: Permission denied" ; return 0}
#.request before cropping
  set text "Crop selected fax pages?"
  set ret [interact label Crop text]
  if !$ret {
    foreach i $items {
      foreach page [glob -nocomplain "$curr_dir/[lindex $file_list $i]"] {
        set fax_name [file tail $page]
        if ![file writable $page] {
          log_entry "$fax_name: Permission denied"
        } else {
          log_entry "$fax_name: Cropping file"
          if {[catch "eval exec {$do_Crop_command < $page > $tmp_file}"] || \
              [catch "eval exec {cat $tmp_file > $page}"]}  \
             {log_entry "$fax_name: Cropping -- failed"} \
        }
      }
    }
  }
  catch "eval exec rm -f $tmp_file"
  do_mmail_mod $curr_mode
  if $ret {listbox_restore $items $active}
}


# Toggle display of multipage faxes

proc do_toggle {} {
  global curr_mode plmwin
  if {"$curr_mode" == "FaxIN" || "$curr_mode" == "FaxOUT" || "$curr_mode" == "DTMF"} \
      {$plmwin.button.update invoke}
}


# Send all spooled faxes via faxrunq

proc do_faxrunq {} {
  global curr_mode do_Faxrunq_command faxrunq_pid faxrunq_finished end_cmd busy
  global plmwin

  set items [listbox_selection]
  set active [listbox_active]
  log_entry "Starting faxrunq. Please wait ..."
  if [stop_play] {
#...disable all buttons and key-bindings
    set busy 1
    for {set i 0} {$i < 7} {incr i} {place forget $plmwin.button.$i}
    place forget $plmwin.button.update
    dis_mode
    $plmwin.button.cancel configure -text "Stop" -command "stop_faxrunq"
    place $plmwin.button.cancel -x 473 -y 2
    act_keys 0
    bind $plmwin <Return> {} 
    bind $plmwin <Left>   {}
    bind $plmwin <Right>  {}
    bind $plmwin <s>      {$plmwin.button.cancel invoke} 
    bind $plmwin <Escape> {$plmwin.button.cancel invoke}
#...start faxrunq-command in the background to enable stop_faxrunq
    set faxrunq_finished 0
    set cmd  [lindex [split $do_Faxrunq_command] 0]
    set args [lrange [split $do_Faxrunq_command] 1 end]
    exec_cmd $cmd $args faxrunq_pid wait
    log_entry "... done"
#...restore buttons and key-bindings
    place forget $plmwin.button.cancel
    act_button
    bind $plmwin <s>      {}
    bind $plmwin <Escape> {$plmwin.mode.exit invoke}
    bind $plmwin <Left>   {prev_mode}
    bind $plmwin <Right>  {next_mode}
    do_mmail_mod $curr_mode
    listbox_restore $items $active
    set busy 0
  }
}


# Stop faxrunq

proc stop_faxrunq {} {
  global faxrunq_pid faxrunq_finished lock_file new_logline

  if !$faxrunq_finished {
    log_entry "Stopping faxrunq"
    set new_logline 1
    catch "kill 19 $faxrunq_pid"
    after 500
    catch "kill 9 $faxrunq_pid"
    set sendfax_pid [read_file -nonewline $lock_file]
    if { "$lock_file" != "" } {catch "kill 2 $sendfax_pid"}
    after 2000
#...If a lockfile remains after killing sendfax, delete it
    if [file exists $lock_file] {
      catch "eval exec rm -f $lock_file"
    }
    set faxrunq_finished 1
  }
}


# Remove all pages of marked spooled faxes

proc do_fremove {} {
  global curr_mode curr_dir file_list text

  set items [listbox_selection]
  set active [listbox_active]
  if {$items == ""} {return 0}
  if ![file writable $curr_dir] \
     {log_entry "$curr_dir: Permission denied" ; return 0}
#.request before deleting
  set text "Delete all pages of the selected faxes?"
  set ret [interact label Delete text]
  if !$ret {
    foreach i $items {
      set job_path [file dirname "$curr_dir/[lindex $file_list $i]"]
      set fax_name [file tail "$job_path"]
      if [file exists $job_path]  {
        if ![file writable $job_path] {
          log_entry "$fax_name: Permission denied"
        } else {
          log_entry "$fax_name: Deleting fax"
          if [catch "eval exec {rm -rf $job_path}" msg] \
             {log_entry "$fax_name: Deleting -- failed: $msg"}
        }
      }
    }
  }
  do_mmail_mod $curr_mode
  if $ret {listbox_restore $items $active}
}


# Suspend marked spooled faxes / respool suspended or already sent faxes

proc do_suspend {} {
  global curr_dir file_list plmwin

  set items [listbox_selection]
  if {$items == ""} {return 0}
  set active [listbox_active]
  if ![file writable $curr_dir] \
     {log_entry "Can't write to $curr_dir" ; return 0}
  foreach i $items {
    set job_path [file dirname "$curr_dir/[lindex $file_list $i]"]
    set fax_name [file tail "$job_path"]
    if [file exists "$job_path/JOB.suspended"] {
      log_entry "$fax_name: Respooling fax"
      if [catch "eval exec {mv $job_path/JOB.suspended $job_path/JOB}"] \
         {log_entry "$fax_name: Respooling -- failed"}
    } elseif [file exists "$job_path/JOB.done"] {
      log_entry "$fax_name: Respooling fax"
      if [catch "eval exec {mv $job_path/JOB.done $job_path/JOB}"] \
         {log_entry "$fax_name: Respooling -- failed"}
    } elseif [file exists "$job_path/JOB"] {
      log_entry "$fax_name: Suspending fax"
      if [catch "eval exec {mv $job_path/JOB $job_path/JOB.suspended}"] \
         {log_entry "$fax_name: Suspending -- failed"}
    }
  }   
  $plmwin.button.update invoke
  listbox_restore $items $active
}


# Add phone-numbers for marked incoming faxes to the faxsender-database

proc do_add_sender {} {
  global curr_mode fax_db faxdb_file file_list sender

  set items [listbox_selection]
  if {$items == ""} {return 0}
  set active [listbox_active]
  create_file $faxdb_file
  if ![file writable $faxdb_file] \
     {log_entry "Can't write to $faxdb_file" ; return 0}
  set changes 0
  foreach line $items {
    set file_name [lindex $file_list $line]
    set file_split [split $file_name "."]
    set phone_no [string range [lindex $file_split 0] 12 50]
    set phone_no [string trim "$phone_no" "_"]
    set phone_no [string trim "$phone_no" "-"]
#...is there already an entry in the fax-database?
    set sender ""
    foreach entry "$fax_db" {
      if ![string compare [lindex "$entry" 0] "$phone_no"] {
        set sender [lrange "$entry" 1 end]
        break
      }
    }
#...ask for full name of sender
    set old_sender "$sender"
    log_entry "Enter full name for $phone_no:"
    if ![interact entry Enter sender] {
      if {[string compare "$sender" ""] && \
          [string compare "$sender" "$old_sender"]} {
        set changes 1
        if [string compare "$old_sender" ""] {
#.........remove old entry from the database
          set old_fax_db "$fax_db"
          set fax_db ""
          foreach entry "$old_fax_db" {
            if [string compare [lindex "$entry" 0] "$phone_no"] {
              lappend fax_db $entry
            }
          }
        }
        lappend fax_db [eval list $phone_no [split $sender]]
        log_entry "$phone_no is $sender"
      } elseif {![string compare "$sender" ""] && \
                 [string compare "$old_sender" ""]} {
#.......remove entry from the database
        set changes 1
        set old_fax_db "$fax_db"
        set fax_db ""
        foreach entry "$old_fax_db" {
          if [string compare [lindex "$entry" 0] "$phone_no"] {
            lappend fax_db $entry
          }
        }
        log_entry "Removing full name for $phone_no"
      }
    }
  }
  if $changes {
#...write new database
    if [catch {open $faxdb_file w} file_id] {
      log_entry "$faxdb_file: error writing file"
    } else {
      foreach entry "$fax_db" {
        if {"$entry" != ""} {puts $file_id "$entry"}
      }
      catch {close $file_id}
    }
  }
  do_mmail_mod $curr_mode
  listbox_restore $items $active
}

