# ------------------------------------------------------------------------------
# File selector box (loosely based on XTeXShell)
# ------------------------------------------------------------------------------

proc select_file {startdir {prompt_text ""}} {
  global fsbfname fsbwin fsbwin_name fsbwin_geo fsboptions
  global normal_font bold_font alt_key old_startdir

  if [winfo exists .filesel] {
    log_entry "Fileselection box is already open!"
    return ""
  }

  if {"$prompt_text" != ""} {log_entry "$prompt_text"}

  set fsbfname [expr {[string compare $startdir "."] ? $startdir : [eval pwd]}]
  set title "Choose existing File"
  set old_startdir $fsbfname

#.generate window
  set fsbwin_name ".filesel"
  set fsbwin_geo  "50x30"
  set fsboptions  "-a -la"
  set newwin [create_topwin fsbwin]
  if ![string compare $newwin ""]  return \
  else {set fsbwin $newwin}

  wm title    $fsbwin $title
  wm iconname $fsbwin "File Selector Box"
  wm minsize  $fsbwin 5 5

  entry     $fsbwin.mask   -relief sunken -textvariable fsbfname \
                           -font $bold_font -exportselection 0
  frame     $fsbwin.but
  scrollbar $fsbwin.scroll -command "$fsbwin.list yview"
  listbox   $fsbwin.list   -yscroll "$fsbwin.scroll set" -relief flat \
                           -width 20 -height 20 \
                           -setgrid yes -font $bold_font -exportselection 0

  pack $fsbwin.mask   -side top    -fill x -padx 4 -pady 3
  pack $fsbwin.but    -side bottom -fill x -padx 2
  pack $fsbwin.scroll -side right  -fill y -padx 4
  pack $fsbwin.list   -side left   -fill both -expand yes -padx 4

#.generate buttons
  button $fsbwin.but.ok     -text "OK"     -width 7 -padx 2 -pady 2 \
    -font $normal_font \
    -command {fsb_command SELECT}
  button $fsbwin.but.cancel -text "Cancel" -width 7 -padx 2 -pady 2 \
    -font $normal_font \
    -command {fsb_command CANCEL}
  button $fsbwin.but.form   -text "Format" \
    -font $normal_font \
    -underline 0 -width 7 -padx 2 -pady 2 -command {fsb_command FORMAT}
  button $fsbwin.but.start  -text "Start"  \
    -font $normal_font \
    -underline 0 -width 7 -padx 2 -pady 2 -command {fsb_command START}
  button $fsbwin.but.home   -text "Home" \
    -font $normal_font \
    -underline 0 -width 7 -padx 2 -pady 2 -command {fsb_command HOME}
  pack $fsbwin.but.ok $fsbwin.but.cancel $fsbwin.but.form \
       $fsbwin.but.start $fsbwin.but.home \
    -side left -padx 2 -pady 2

  bind $fsbwin      <$alt_key-f>      {fsb_command FORMAT}
  bind $fsbwin      <$alt_key-s>      {fsb_command START}
  bind $fsbwin      <$alt_key-h>      {fsb_command HOME}
  bind $fsbwin      <Escape>          {fsb_command CANCEL}
  bind $fsbwin.mask <Return>          {fsb_command SELECT}
  bind $fsbwin.list <Return>          {fsb_command SELECT}
  bind $fsbwin.list <Double-Button-1> {fsb_command SELECT}

#.read file list
  fsb_getlist

#.wait for OK or cancel
  tkwait visibility $fsbwin.list
  focus $fsbwin.list
  tkwait window $fsbwin
  return $fsbfname
}


# Handle user input of the file selector box

proc fsb_command {Mode} {
  global fsbwin fsbfname startdir old_startdir fsboptions

  if ![string compare $Mode "CANCEL"] {
#...user requested CANCEL
    set fsbfname ""
    destroy $fsbwin
    return
  } elseif ![string compare $Mode "FORMAT"] {
    set newoptions [list [lindex $fsboptions 1] [lindex $fsboptions 0]]
    set fsboptions $newoptions
    fsb_getlist
    return
  } elseif ![string compare $Mode "START"] {
#...user requested START
    set fsbfname $old_startdir
    set Mode "RETURN"
  } elseif ![string compare $Mode "HOME"] {
#...user requested HOME
    set oldfsbfname [eval pwd]
    cd
    set fsbfname [eval pwd]
    cd $oldfsbfname
    set Mode "RETURN"
  } elseif ![string compare $Mode "SELECT"] {
#...user pressed double button 1 or return
    if ![string compare [$fsbwin.list curselection] ""] return
    set str [$fsbwin.list get [$fsbwin.list curselection]]
    set str [lindex $str end]
    if ![string compare $str "./"] {
      fsb_getlist
      return
    }
    if ![string compare $fsbfname "./"] {
      set fsbfname $str
    } else {
      set fsbfname [string trimright $fsbfname "/"]
      if ![string compare $str "../"] {
        if ![string compare $fsbfname ""] {set fsbfname "/"} \
        else {set fsbfname [file dirname $fsbfname]}
      } else {
        set fsbfname "$fsbfname/$str"
      }
    }
    set Mode "RETURN"
  }
#.user requested RETURN
  if ![string compare $Mode "RETURN"] {
    if {[file isdirectory $fsbfname]} {
      fsb_getlist
      set startdir $fsbfname
      return
    }
  }
#.OK, user has selected a filename which is in fsbfname.
  if [file exists $fsbfname] {
    destroy $fsbwin
    return
  }
}


# Clear file selector listbox. Then fill listbox with directory entries

proc fsb_getlist {} {
  global fsbfname fsboptions fsbwin

  $fsbwin.list delete 0 end
  set filelist [exec ls [lindex $fsboptions 0] $fsbfname]
  foreach i [split $filelist "\n"] {
#...append "/" to each directory name
    set filename $i
    set filename [lindex $filename end]
    if [file isdirectory $fsbfname/$filename] {set i "$i/"}
#...don't display current directory "." or "total ..."
    if {[string compare $filename "."] && [llength [split $i]] != 2} {
      $fsbwin.list insert end $i
    }
  }
  if {[string compare "/" [string index $fsbfname \
                            [expr [string length $fsbfname]-1]]]} {
    set fsbfname "$fsbfname/"
  }
}

