# ------------------------------------------------------------------------------
# Create the main window
# ------------------------------------------------------------------------------

proc main_window {} {
  global plmwin plmwin_name plmwin_geo wm_title mmail_mode curr_mode
  global normal_font fixed_font list_font
  global color_disabled color_input color_selection alt_key
  global faxdb_file faxrunq_finished connect_finished end_cmd
  global new_logline play_pid update_time fax_db shift ctrl

#.generate main window
  set plmwin_name ".palmail"
  set plmwin_geo  "540 354"
  set plmwin [create_topwin plmwin]
  wm title    $plmwin $wm_title
  wm iconname $plmwin "PalMail"
  wm minsize  $plmwin 540 354
  wm maxsize  $plmwin 540 354

#.search for possible modes
  set i 0
  while {$i < [llength $mmail_mode]} {
    set test_path [lindex [lindex $mmail_mode $i] 2]
    set test_file [lindex [lindex $mmail_mode $i] 3]
    if [expr ![file readable $test_path]]  {set del_mode 1} \
    elseif {[string first "*" "$test_file"] >= 0} {set del_mode 0} \
    else {set del_mode ![file readable "$test_path/$test_file"]}
    if $del_mode {
#.....$test_path/$test_file doesn't exist: remove corresponding mode
      if {$i > 0} {
        if {$i == [expr [llength $mmail_mode] - 1]} {
          set mmail_mode [lrange $mmail_mode 0 [expr $i - 1]]
        } else {
          set mmail_mode [concat [lrange $mmail_mode 0 [expr $i - 1]] \
                                 [lrange $mmail_mode [expr $i + 1] end]]
        }
      } else {
        if {[llength $mmail_mode] == 1} {
          puts "No modes available. Check your configuration."
          exit 1
        } else {set mmail_mode [lrange $mmail_mode [expr $i + 1] end]}
      }
    } else {set i [expr $i + 1]}
  }

#.read fax-database
  set fax_db {}
  if {[file readable $faxdb_file] && ![catch {open $faxdb_file r} file_id]} {
    while {![eof $file_id]} {
      gets $file_id line
      lappend fax_db [lrange [split "$line"] 0 end]
    }
    catch {close $file_id}
  }

#.create mode buttons
  pack [frame $plmwin.mode -width 540 -height 32]
  set but_x 4
  foreach i $mmail_mode {
    set mode_name [string tolower [lindex $i 0]]
    button $plmwin.mode.$mode_name -text [lindex $i 0] -underline 0 \
      -font $normal_font \
      -width 7 -padx 2 -pady 2 -disabledforeground $color_disabled \
      -command "do_mmail_mod [lindex $i 0]"
    place $plmwin.mode.$mode_name -x $but_x -y 3
    incr but_x 67
    eval "bind $plmwin [lindex $i 1] {$plmwin.mode.$mode_name invoke}"
  }
  button $plmwin.mode.exit -text Exit -underline 0 \
    -font $normal_font \
    -width 7 -padx 2 -pady 2 -disabledforeground $color_disabled \
    -command do_exit
  place $plmwin.mode.exit -x $but_x -y 3
  bind $plmwin <$alt_key-e> {$plmwin.mode.exit invoke}
  bind $plmwin <Escape> {$plmwin.mode.exit invoke}
  bind $plmwin <Left> {prev_mode}
  bind $plmwin <Right> {next_mode}

#.file/item display
  pack [frame $plmwin.display -width 540 -height 148]
  listbox $plmwin.display.listbox -selectmode multiple -width 71 -height 7 \
    -font $list_font -highlightthickness 0 -background $color_input \
    -selectbackground $color_selection -selectforeground $color_input \
    -xscrollcommand "$plmwin.display.xscroll set" \
    -yscrollcommand "$plmwin.display.yscroll set"
  place $plmwin.display.listbox -x 6 -y 2 -width 506
  scrollbar $plmwin.display.yscroll -orient vertical -width 14 \
    -command "$plmwin.display.listbox yview"
  place $plmwin.display.yscroll -x 514 -y 0 -height 127
  scrollbar $plmwin.display.xscroll -orient horizontal -width 14 \
    -command "$plmwin.display.listbox xview"
  place $plmwin.display.xscroll -x 4 -y 127 -width 510
  $plmwin.display.yscroll set 0 7 0 0
  set shift 0
  set ctrl 0

#.status bar
  pack [frame $plmwin.status -width 540 -height 27]
  label $plmwin.status.label1 -text {Process information:} -font $normal_font
  place $plmwin.status.label1 -x 3 -y 5
  label $plmwin.status.label2 -text "Multipage" -underline 0 -font $normal_font
  place $plmwin.status.label2 -x 452 -y 5
  checkbutton $plmwin.status.button -command do_toggle \
    -variable show_multi -relief flat
  place $plmwin.status.button -x 514 -y 3

#.logging
  pack [frame $plmwin.log -width 540 -height 115]
  text $plmwin.log.text -height 8 -padx 2 -font $fixed_font \
    -state disabled -relief sunken \
    -yscrollcommand "$plmwin.log.yscroll set"
  place $plmwin.log.text -x 4 -y 2 -width 510
  scrollbar $plmwin.log.yscroll -orient vertical -width 14 \
    -command "$plmwin.log.text yview"
  place $plmwin.log.yscroll -x 514 -y 2 -height 114
  $plmwin.log.text yview 0

#.create command buttons, but don't place them now
  pack [frame $plmwin.button -width 540 -height 32]
  for {set i 0} {$i < 7} {incr i} {
    button $plmwin.button.$i -font $normal_font \
      -width 7 -padx 2 -pady 2 -disabledforeground $color_disabled
  }
  button $plmwin.button.update -font $normal_font \
    -width 7 -padx 2 -pady 2 -disabledforeground $color_disabled
#.also create a label, an entry widget and two extra buttons in this frame
  label $plmwin.button.label -font $normal_font -anchor w
  entry $plmwin.button.entry -relief sunken -width 55 \
    -background $color_input -highlightthickness 0 -font $normal_font
  button $plmwin.button.action -font $normal_font \
    -width 7 -padx 2 -pady 2 -disabledforeground $color_disabled
  button $plmwin.button.cancel -font $normal_font \
    -width 7 -padx 2 -pady 2 -disabledforeground $color_disabled

  set curr_mode [lindex [lindex $mmail_mode 0] 0]
  act_button

  set faxrunq_finished 1
  set connect_finished 1
  set end_cmd 1
  set new_logline 1
  set play_pid 0

  std_keys
  log_entry "Starting session"
  do_mmail_mod $curr_mode
  tkwait visibility $plmwin.display.listbox
  focus $plmwin.display.listbox
}

# ------------------------------------------------------------------------------
# Buttons and standard bindings
# ------------------------------------------------------------------------------

proc std_keys {} {
  global shift_up shift_down alt_key plmwin

#.Uncomment the following line if you want to find key-names for your machine
# bind $plmwin <Any-Key> {puts {Keyname=%K}}

  bind $plmwin <KeyPress-Shift_L>   {set shift 1}
  bind $plmwin <KeyRelease-Shift_L> {set shift 0}
  bind $plmwin <KeyPress-Control_L>   {set ctrl 1}
  bind $plmwin <KeyRelease-Control_L> {set ctrl 0}
  bind $plmwin <$shift_up>   {scroll_logbox -1}
  bind $plmwin <$shift_down> {scroll_logbox  1}
  bind $plmwin <Tab>  {}
  bind $plmwin <Shift-Tab>  {}
  bind $plmwin <Up>   {if $shift {scroll_logbox -1}}
  bind $plmwin <Down> {if $shift {scroll_logbox  1}}
  bind $plmwin <Key-space> {listbox_select}
  bind $plmwin.display.listbox <Button-1> {listbox_select 1 "%y"}
  bind $plmwin.display.listbox <Double-Button-1> {listbox_double_click}
  bind $plmwin.display.listbox <Button1-Motion> {listbox_drag %y}
  bind $plmwin <$alt_key-m> {$plmwin.status.button invoke}
}


# Set buttons for a new mode and update the listbox

proc do_mmail_mod {newmode} {
  global curr_mode curr_dir curr_file mmail_mode plmwin
  global color_foreground color_disabled

#.disable all buttons and their key-shortcuts
  act_keys 0
  for {set i 0} {$i < 7} {incr i} \
      {$plmwin.button.$i configure -state disabled -relief flat -text "" \
                                   -command ""}
  $plmwin.button.update  configure -state disabled -relief flat -text "" \
                                   -command ""
#.enable buttons for the mode
  foreach i $mmail_mode {
    if {[lindex $i 0] == $newmode} {
      set curr_dir  [lindex $i 2]
      set curr_file [lindex $i 3]
      set curr_mode $newmode
      $plmwin.mode.[string tolower [lindex $i 0]] configure \
          -state disabled -text [lindex $i 0]
      foreach but [lindex $i 4] {
        set bind_char [string first \
                        [string toupper [string index [lindex $but 3] 1]] \
                        [lindex $but 1]]
        $plmwin.button.[lindex $but 0] configure \
            -state normal -relief raised -underline $bind_char \
            -command [lindex $but 2] -text [lindex $but 1]
      }
    } else {
      $plmwin.mode.[string tolower [lindex $i 0]] configure \
          -state normal -text [lindex $i 0]
    }
  }
#.special case: Exit
  if {$newmode == "Exit"} {
    set curr_mode $newmode
    $plmwin.mode.exit configure -foreground $color_disabled -text Exit
    eval "bind $plmwin <Return> {$plmwin.mode.exit invoke}"
    $plmwin.display.listbox delete 0 end
  } else {
    $plmwin.mode.exit configure -foreground $color_foreground -text Exit
    act_keys 1
    $plmwin.button.update invoke
  }

#.adjust the checkbox
  if {$newmode == "VocIN" || $newmode == "VocOUT" || \
      $newmode == "DTMF"  || $newmode == "Ring" } {
    $plmwin.status.label2 configure -text "use Modem" -underline 4
    place $plmwin.status.label2 -x 440 -y 5
    $plmwin.status.button configure -variable use_modem
  } else {
    $plmwin.status.label2 configure -text "Multipage" -underline 0
    place $plmwin.status.label2 -x 452 -y 5
    $plmwin.status.button configure -variable show_multi
  }
}


# Disable or enable key-bindings for the current mode

proc act_keys {value} {
  global curr_mode mmail_mode plmwin

  if {$value} {bind $plmwin <Return> {$plmwin.button.0 invoke}}
  foreach i $mmail_mode {
    if {[lindex $i 0] == $curr_mode} {
      foreach but [lindex $i 4] {
        if $value {eval "bind $plmwin [lindex $but 3] {
          $plmwin.button.[lindex $but 0] invoke}"
        } else {
          eval "bind $plmwin [lindex $but 3] {}"
        }
      }
    }
  }
}


# Disable all mode-buttons

proc dis_mode {} {
  global curr_mode mmail_mode color_foreground plmwin

  act_keys 0
  foreach i $mmail_mode {
    if {[lindex $i 0] == $curr_mode} {
      $plmwin.mode.[string tolower [lindex $i 0]]  \
          configure -state disabled -text [lindex $i 0]
    } else {
      $plmwin.mode.[string tolower [lindex $i 0]]  \
          configure -state disabled -text ""
    }
  }
  $plmwin.mode.exit configure -foreground $color_foreground -text ""
}


# Activate command-buttons

proc act_button {} {
  global plmwin

  set but_x 4
  for {set i 0} {$i < 7} {incr i} {
    place $plmwin.button.$i -x $but_x -y 2
    incr but_x 67
  }
  place $plmwin.button.update -x $but_x -y 2
  act_keys 1
}


# Request user-interaction

proc interact {mode button_text variable} {
  global busy ret text $variable color_input plmwin

  set busy 1
#.remove command buttons
  for {set i 0} {$i < 7} {incr i} {place forget $plmwin.button.$i}
  place forget $plmwin.button.update
  dis_mode
  act_keys 0
  bind $plmwin <Left> {}
  bind $plmwin <Right> {}
#.place the label, the entry and the extra buttons instead
  if {$mode == "label"} {
    $plmwin.button.label configure -textvariable $variable
    place $plmwin.button.label -x 3 -y 6
  } else {
    $plmwin.button.entry configure -textvariable $variable
    place $plmwin.button.entry -x 6 -y 6
    focus $plmwin.button.entry
  }
  $plmwin.button.action configure -text $button_text \
    -command "end_interact 0"
  place $plmwin.button.action -x 406 -y 2
  $plmwin.button.cancel configure -text "Cancel" \
    -command "end_interact 1"
  place $plmwin.button.cancel -x 473 -y 2
  bind $plmwin <Return> {$plmwin.button.action invoke} 
  bind $plmwin <Escape> {$plmwin.button.cancel invoke}
  set ret -1
  tkwait variable ret
  focus $plmwin.display.listbox
  act_keys 1
  bind $plmwin <Left>  {prev_mode}
  bind $plmwin <Right> {next_mode}
  bind $plmwin <Escape> {$plmwin.mode.exit invoke}
  act_button
  set busy 0
  return $ret
}


# End of user-interaction

proc end_interact {mode} {
  global plmwin ret

  place forget $plmwin.button.label
  place forget $plmwin.button.entry
  place forget $plmwin.button.action
  place forget $plmwin.button.cancel
  set ret $mode
}


# Circulate to the next mode on the right

proc next_mode {} {
  global curr_mode mmail_mode

  set nr 1
  set new_mode {}
  foreach i $mmail_mode {
    if {[lindex $i 0] == $curr_mode} {
      set new_mode [lindex [lindex $mmail_mode $nr] 0]
      if {"$new_mode" == ""} {set new_mode Exit}
      break
    }
    incr nr
  }
  if {$new_mode == ""} {set new_mode [lindex [lindex $mmail_mode 0] 0]}
  do_mmail_mod $new_mode
}


# Circulate to the next mode on the left

proc prev_mode {} {
  global curr_mode mmail_mode

  set nr 0
  set new_mode {}
  foreach i $mmail_mode {
    if {[lindex $i 0] == $curr_mode} {
      if !$nr {set new_mode Exit} \
      else {set new_mode [lindex [lindex $mmail_mode [expr "$nr - 1"]] 0]}
      break
    }
    incr nr
  }
  if {$new_mode == ""} \
     {set new_mode [lindex [lindex $mmail_mode [expr [llength $mmail_mode] - 1]] 0]}
  do_mmail_mod $new_mode
}


# Scroll logbox

proc scroll_logbox {lines} {
  global plmwin
  $plmwin.log.text yview scroll $lines units
}


# Get active listbox-element

proc listbox_active {} {
  global plmwin
  return [$plmwin.display.listbox index active]
}


# Get listbox-selection

proc listbox_selection {} {
  global plmwin
  return [$plmwin.display.listbox curselection]
}


# Clear listbox-selection

proc listbox_select_clear {} {
  global plmwin
  $plmwin.display.listbox select clear 0 end
}


# Restore listbox-selection

proc listbox_restore {items {active end}} {
  global plmwin
  $plmwin.display.listbox select clear 0 end
  foreach i $items {$plmwin.display.listbox select set $i}
  $plmwin.display.listbox activate $active
}


# Select or deselect a listbox-entry

proc listbox_select {{mouse "0"} {y "0"}} {
  global shift ctrl last_elem plmwin

  set select 1
  if $mouse {
    set act [$plmwin.display.listbox nearest $y]
  } else {
    set act [$plmwin.display.listbox index active]
  }
  if {!$ctrl && !$shift} {
    if $mouse {
      if {[lsearch [listbox_selection] $act] >= 0} {
        if {[llength [listbox_selection]] > 1} {
          set mode clear
        } else {set mode set}
      } else {
        set mode clear
      }
    } else {set mode set}
    listbox_select_clear
    $plmwin.display.listbox select $mode $act
  }
  if {$ctrl && $mouse} {
    if {[lsearch [listbox_selection] $act] >= 0} {
      set mode clear
    } else {set mode set}
    $plmwin.display.listbox select $mode $act
  }
  if {$shift && $last_elem != -1} {
#...adjust selection
    if {[lsearch [listbox_selection] $last_elem] >= 0} {
      set mode set
    } else {set mode clear}
    $plmwin.display.listbox select $mode $act $last_elem
  }
  set last_elem $act
}


# Set selection for double-clicks and invoke default-command

proc listbox_double_click {} {
  global last_elem shift plmwin

  $plmwin.display.listbox select set $last_elem
  $plmwin.button.0 invoke
#.The selection will be toggled again with the second button-bress
  if !$shift {$plmwin.display.listbox select clear $last_elem}
}


# Select/deselect listbox-entries while dragging the mouse

proc listbox_drag {y} {
  global last_elem plmwin

  if {[lsearch [listbox_selection] $last_elem] >= 0} {
    set mode set
  } else {set mode clear}
  $plmwin.display.listbox select $mode \
      [$plmwin.display.listbox nearest $y] $last_elem
}

# ------------------------------------------------------------------------------
# Logging
# ------------------------------------------------------------------------------

proc verb_time {clock_val} {
  global weekdays time_fmt tcl_version
  if {$tcl_version >= 7.5} {
    set weekday [lindex $weekdays [clock format $clock_val -format %w]]
    set time_string "$weekday[clock format $clock_val -format $time_fmt]"
  } else {
    set weekday [lindex $weekdays [fmtclock $clock_val %w]]
    set time_string "$weekday[fmtclock $clock_val $time_fmt]"
  }
}

proc log_time {} {
  global tcl_version
  if {$tcl_version >= 7.5} {
    return [clock format [clock seconds] -format "%T: "]
  } else {
    return [fmtclock [getclock] "%T: "]
  }
}

proc log_newline {} {
  global plmwin argv0
  if ![string compare [file tail $argv0] "PalFax"] {
    puts ""
  } else {
#...don't put '\n' at the beginning of the logbox
    if {[$plmwin.log.text get "end - 2 char"] == "\n"} return
    if {[$plmwin.log.text get end] != "\n"} {
      $plmwin.log.text insert end "\n"
      $plmwin.log.text mark set current "current + 1 lines"
    }
  }
}

proc log_enter {args} {
  global plmwin argv0
  if ![string compare [file tail $argv0] "PalFax"] {
    if {$args == ""} {puts ""} \
    else {puts -nonewline [join $args]}
  } else {
    if {$args == ""} {$plmwin.log.text insert end "\n"} \
    else {$plmwin.log.text insert end [join $args]}
    set lines [llength [split [join $args] "\n"]]
    $plmwin.log.text mark set current "current + $lines lines"
  }
}


proc log_event {text} {
  global new_logline plmwin argv0 tcl_version

  if ![string compare [file tail $argv0] "PalFax"] {
    if $new_logline {
      log_newline
      set new_logline 0
    }
    if {[string index "$text" [expr [llength $text]-1]] == "\n"} {
      set new_logline 1
      set text [string trimright "$text" "\n"]
    }
    puts -nonewline "$text"
  } else {
    if {$tcl_version >= 7.5} {
      set time [clock format [clock seconds] -format "%T: "]
    } else {
      set time [fmtclock [getclock] "%T: "]
    }
    $plmwin.log.text configure -state normal
    if $new_logline {
      log_newline
      set text "$time$text"
      set new_logline 0
    }
    if {[string index "$text" [expr [llength $text]-1]] == "\n"} {
      set new_logline 1
      set text [string trimright "$text" "\n"]
    }
    regsub -all "\n" "$text" "\n$time" new_text
    $plmwin.log.text insert end "$new_text"
    $plmwin.log.text configure -state disabled
    $plmwin.log.text yview -pickplace end
  }
  update idletasks
}


proc log_entry {args} {
  global new_logline
  set new_logline 1
  log_event [join $args]
}

