# ------------------------------------------------------------------------------
# Listbox-Update-Commands
# ------------------------------------------------------------------------------

# Insert an entry into file_list and listbox

proc insert_list {file_name file_text {hint ""}} {
  global file_list hint_list new_list plmwin

  if {$new_list == "TRUE"} {
    set file_list [list "$file_name"]
    set hint_list [list "$hint"]
    set new_list "FALSE"
  } else {
    lappend file_list "$file_name"
    lappend hint_list "$hint"
  }
  $plmwin.display.listbox insert end "$file_text"
}


# Automatical update

proc auto_update {} {
  global busy curr_mode update_time plmwin

#.don't update if PalMail is busy
  if {!$busy && $curr_mode == "Ring"} {
    set items [listbox_selection]
    set active [listbox_active]
    $plmwin.button.update invoke
    listbox_restore $items $active
  }
  after ${update_time}000 {auto_update}   
}


proc update_start {} {
  global new_list busy plmwin

  set busy 1
#.create new listbox
  $plmwin.display.listbox delete 0 end
  set new_list "TRUE"
  $plmwin.button.update configure -state active -relief sunken
  update idletasks
}


proc update_end {} {
  global last_elem busy plmwin

# Display last line of the listbox
  $plmwin.display.listbox yview end
  $plmwin.display.listbox activate end
  set last_elem -1
  $plmwin.button.update configure -state normal -relief raised
  update idletasks
  set busy 0
}


# Update the directory-listbox for VocIN and FaxIN

proc do_VocFaxIN_update {} {
  global curr_mode curr_dir curr_file fax_db show_multi
  global VocIN_mask1 VocIN_mask2 VocIN_mask3

  if ![file readable $curr_dir] {log_entry "Can't read $curr_dir" ; return 0}
  update_start
  set match [lsort -command file_sort [glob -nocomplain $curr_dir/$curr_file]]
#.construct listbox-entry
  if {$curr_mode == "VocIN"} {
    foreach f $match {
      if ![file isdirectory $f] {
        set file_name [file tail $f]
        set file_text [verb_time [file mtime $f]]
        foreach mask [list $VocIN_mask1 $VocIN_mask2 $VocIN_mask3 *] {
          if [string match $mask $file_name] {
            set ign_chars [string length [string trimright $mask "*"]]
            append file_text " [string range $file_name $ign_chars end]"
            break
          }
        }
        insert_list "$file_name" "$file_text"
      }
    }
  } else {
#...FaxIN
    set curr_fax ""
    foreach f $match {
      if ![file isdirectory $f] {
#.......show file-time, sender and page-no
        set file_name [file tail $f]
        set file_split [split $file_name "."]
        set file_page "/ [lindex $file_split 1]"
        set file_date [verb_time [file mtime $f]]
        set file_from [string range [lindex $file_split 0] 12 50]
        set file_from [string trim "$file_from" "_"]
        set file_from [string trim "$file_from" "-"]
#.......detect fax resolution
        if {[string range "$file_name" 0 1] == "fn"} {set hint "n"} \
        else {set hint "f"}
        if $show_multi  {
#.........display each fax only once
          if {[lindex $file_split 0] != "$curr_fax"} {
            set curr_fax [lindex $file_split 0]
#...........look for full name of sender in the fax-database
            foreach entry "$fax_db" {
              if ![string compare [lindex "$entry" 0] "$file_from"] \
                 {set file_from [lrange "$entry" 1 end] ; break}
            }
#...........count pages
            set pages [llength [glob -nocomplain "$curr_dir/[lindex $file_split 0].*"]]
            if {"$pages" == "1"} {set file_page ""} \
            else                 {set file_page "($pages)"}
            set file_text "$file_date $file_from $file_page"
            insert_list "[lindex $file_split 0].*" "$file_text" "$hint"
          }
        } else {
#.........display each page separately
#.........look for full name of sender in the fax-database
          foreach entry "$fax_db" {
            if ![string compare [lindex "$entry" 0] "$file_from"] \
               {set file_from [lrange "$entry" 1 end] ; break}
          }
#.........if there is only 1 page, don't display the page-number
          if {$file_page == "/ 01" && \
              ![file exists $curr_dir/[lindex $file_split 0].02]} {set file_page ""}
          set file_text "$file_date $file_from $file_page"
          insert_list "$file_name" "$file_text" "$hint"
        }
      }
    }
  }
  update_end
}


# Update the directory-listbox for VocOUT

proc do_VocOUT_update {} {
  global curr_dir curr_file index_db

  if ![file readable $curr_dir] {log_entry "Can't read $curr_dir" ; return 0}
  update_start
#.read and sort file names
  read_Index
  set files [lsort [glob -nocomplain $curr_dir/$curr_file]]
  foreach f $files {
    if ![file isdirectory $f] {
      set file_name "[file tail $f]"
#.....don't show files 'Index' and 'beep'
      if {$file_name != "Index" && $file_name != "beep"} {
#.......look for filename in the Index-database
        if {[lsearch -exact "$index_db" "$file_name"] >= 0} {set hint "*"} \
        else {set hint "-"}
        set file_text "$hint $file_name"
        insert_list "$file_name" "$file_text" "$hint"
      }
    } 
  }
  update_end
}


# Update the directory-listbox for FaxOUT

proc do_FaxOUT_update {} {
  global curr_dir curr_file show_multi

  if ![file readable $curr_dir] {log_entry "Can't read $curr_dir" ; return 0}
  update_start
#.read and sort file names
  set files [lsort -command file_job_sort [glob -nocomplain $curr_dir/$curr_file]]
  foreach f $files {
    if [file isdirectory $f] {
      set file_name "[file tail $f]"
      set job_path "$curr_dir/$file_name"
#.....status of the fax job
      if [file exists "$job_path/JOB.done"] {
        set job_file $job_path/JOB.done
        set job_status "d:"
      } elseif [file exists "$job_path/JOB.suspended"] {
        set job_file $job_path/JOB.suspended
        set job_status "s:"
      } elseif [file exists "$job_path/JOB.locked"] {
        set job_file $job_path/JOB.locked
        set job_status "L:"
      } elseif [file exists "$job_path/JOB"] {
        set job_file $job_path/JOB
        set job_status "q:"
      } else {set job_file ""}
#.....read job-file, start with empty values
      if {$job_file != "" && ![catch {open $job_file r} file_id]} {
        set job_phone ""
        set job_user ""
        set job_pages ""
        set job_input ""
        set job_verbose_to ""
        set job_normal_res "0"
        while {![eof $file_id]} {
          gets $file_id line
          set job_[lindex [split "$line"] 0] [lrange [split "$line"] 1 end]
        }
        catch {close $file_id}
        set job_date [verb_time [file mtime $job_file]]
        if {$job_verbose_to != ""} {set job_phone $job_verbose_to}
#.......process pages (ignore extra space after 'pages ' in the job-file)
        set pages [lrange [split "$job_pages"] 1 end]
        if $show_multi {
          set page_name "$file_name/f*"
          if {[llength "$pages"] == "1"} {set page ""} \
          else                           {set page "([llength $pages])"}
          set page_text \
              "$job_status $job_date $job_user to $job_phone, $job_input $page"
          if {"$job_normal_res" == ""} {set hint "n"} \
          else                         {set hint "f"}
          insert_list "$page_name" "$page_text" "$hint"
        } else {
          foreach page $pages {
            if [file exists $job_path/$page] {
              set page_name "$file_name/$page"
              if {[llength "$pages"] != "1"} {
                set page "/ [string range [lindex [split "$page" "."] 0] 1 end]"
              } else {set page ""}
              set page_text \
                  "$job_status $job_date $job_user to $job_phone, $job_input $page"
              if {"$job_normal_res" == ""} {set hint "n"} \
              else                         {set hint "f"}
              insert_list "$page_name" "$page_text" "$hint"
              set job_status "  "
            }
          }
        }
      } else {insert_list "$file_name/." "?: $file_name" ""}
    }
  } 
  update_end
}


# Update the ringlog-listbox

proc do_ringlog_update {} {
  global curr_dir curr_file plmwin

  if ![file readable $curr_dir] \
     {log_entry "Can't read $curr_dir" ; return 0}
  if ![file readable $curr_dir/$curr_file] \
     {log_entry "Can't read $curr_dir/$curr_file" ; return 0}
  update_start
  update_ringlog
#.call external function
  read_ringlog_c
  $plmwin.display.listbox xview 0
  update_end
}


# Update the file-listbox (Connect-mode)

proc do_file_update {} {
  global curr_dir curr_file plmwin

  if ![file readable $curr_dir] \
     {log_entry "Can't read $curr_dir" ; return 0}
  if ![file readable $curr_dir/$curr_file] \
     {log_entry "Can't read $curr_dir/$curr_file" ; return 0}
  update_start
  set file_id [open $curr_dir/$curr_file "r"]
  while {![eof $file_id]} {
    gets $file_id line
#...ignore last empty line
    if {![eof $file_id] || $line != ""} {
      $plmwin.display.listbox insert end "$line"
    }
  }
  catch {close $file_id}
  $plmwin.display.listbox xview 0
  update_end
}


# Command for lsort based upon file modification date (VocIN / FaxIN)

proc file_sort {fname1 fname2} {
  return [string compare [file mtime $fname1] [file mtime $fname2]]
}


# Command for lsort based upon modification date of the Job-file (FaxOUT)

proc file_job_sort {fname1 fname2} {
#.find job_files
  set job_file1 [lindex [glob -nocomplain "$fname1/JOB*"] 0]
  if {$job_file1 == ""} {set job_file1 $fname1}
  set job_file2 [lindex [glob -nocomplain "$fname2/JOB*"] 0]
  if {$job_file2 == ""} {set job_file2 $fname2}
  return [string compare [file mtime $job_file1] [file mtime $job_file2]]
}

