# ------------------------------------------------------------------------------
# Voice-Commands
# ------------------------------------------------------------------------------

set is_recording 0

# Stop recording / playing voice-messages

proc exec_stop {mode} {
  global play_pid
  if $play_pid {
    catch "kill $mode $play_pid"
  }
}


# Stop recording / playing voice-messages

proc do_stop {} {
  global is_recording

  if {$is_recording} {
    set stop_signal 2
  } else {
    set stop_signal 10
  }

  log_entry "Stop Rec/Play"
  exec_stop $stop_signal
  if $is_recording {
    set is_recording 0
    do_VocOUT_update
  }
}


# Try do stop play/record. Return 1 if succeeded.

proc stop_play {} {
  global lock_file play_pid tcl_version

  if !$play_pid {return 1}
  exec_stop 2
  if {$tcl_version >= 7.5} {
    global f_mkCmd
    catch "close $f_mkCmd"
  } else {
    global i_cmd o_cmd e_cmd
    catch "close $i_cmd"
    catch "close $o_cmd"
    catch "close $e_cmd"
  }
  after 200
  if [file exists $lock_file] {after 3000}
  if [file exists $lock_file] {
    log_entry "Device locked. Please try later again."
    return 0
  } else {return 1}
}


# Play marked recorded voice-messages

proc do_play {} {
  global curr_dir file_list use_modem play_cmd

  set items [listbox_selection]
  if {$items == ""} {return 0}
#.test the selected items as files
  set files ""
  foreach i $items {
    if { [lindex $file_list $i] != "" } {
      set f "$curr_dir/[lindex $file_list $i]"
      if ![file readable $f] {
        set n "[string range $f [expr [string length $curr_dir]+1] end]"
        log_entry "Can't read $n"
      } else {lappend files $f}
    }
  }
  if ![llength $files] {return 0}
  set n1 [file tail [lindex $files 0]]
  set n2 [file tail [lindex $files [expr [llength $files] - 1]]]
  if {[llength $files] == 1} {log_entry "$n1: Playing file"} \
  else {log_entry "$n1 .. $n2: Playing files"}
  if ![stop_play] {return 0}
  if $use_modem {
    set args [concat play modem $files]
  } else {
    set args [concat play soundcard $files]
  }
  exec_cmd $play_cmd $args play_pid
}


# Ask for a filename and record a message

proc do_record {} {
  global curr_mode curr_dir recname plmwin is_recording use_modem play_cmd

  if ![file writable $curr_dir] \
     {log_entry "Can't write to $curr_dir" ; return 0}
#.clear selections
  $plmwin.button.update invoke
  if {![interact entry Record recname] && "$recname" != ""} {
    if {"$recname" == "bell" || "$recname" == "Index"} {
      log_entry "Record: Can't use reserved name '$recname'"
    } else {
      log_entry "$recname: Record file"
      if [stop_play] {
        if $use_modem {
          set args [concat rec modem $curr_dir/$recname]
        } else {
          set args [concat rec soundcard $curr_dir/$recname]
        }
        if [exec_cmd $play_cmd $args play_pid] {
          set is_recording 1
#.........wait a moment until the modem is (hopefully) ready
          if $use_modem {after 3000} \
          else          {after 300}
          log_entry "$recname: Press Stop to stop recording"
        }
      }
    }
  }
  do_mmail_mod $curr_mode
}


# Export a voice-message

proc do_export_voc {} {
  global do_ToAu_command do_ToWav_command do_ToVoc_command
  global do_ToLin_command do_ToBasic_command do_ToRmd_command
  global uservoc_dir curr_mode curr_dir newname text file_list vocout_dir

  set items [listbox_selection]
  if {$items == ""} {return 0}
  set active [listbox_active]
  foreach line $items {
    set line_name [lindex $file_list $line]
    set filename "$curr_dir/$line_name"
    log_entry "Choose output file for $line_name"
    set newname "$uservoc_dir/$line_name"
    if {![interact entry Export newname] && "$newname" != ""} {
      if [file isdirectory $newname] {
        log_entry "$newname is a directory"
        return
      }
      set out_dir [file dirname $newname]
      if {"$out_dir" == "$curr_dir"} {
        log_entry "Output files in $curr_dir are not allowed"
        return
      }
      if ![file isdirectory $out_dir] {
        log_entry "Directory $out_dir doesn't exist"
        return
      }
      set uservoc_dir [file dirname $newname]
#.....determine type of output soundfile (default is Rmd)
      set ext [file extension $newname]
      if {"$ext" == ".au"} {set sound_type Au} \
      elseif {"$ext" == ".wav"} {set sound_type Wav} \
      elseif {"$ext" == ".voc"} {set sound_type Voc} \
      elseif {"$ext" == ".lin"} {set sound_type Lin} \
      elseif {"$ext" == ".bas"} {set sound_type Basic} \
      else {set sound_type Rmd}
      if {"$out_dir" == "$vocout_dir"} {
        if {"$sound_type" != "Rmd"} {
          log_entry "Files in $out_dir must be of type rmd"
          return
        }
      }
#.....Check if there is already a file with this name
      if [file exists "$newname"] {
        set text "Overwrite existing file $newname ?"
        set do_it ![interact label OK text]
      } else {set do_it 1}
      if $do_it {
        log_entry "Converting $filename to $newname"
        set cmd "do_To${sound_type}_command"
        eval set cmd "$$cmd"
        log_entry "$cmd"
        if [catch "eval exec {cat $filename | $cmd > $newname}" output] {
          log_entry "$output"
        }
      }
    }
  }
  do_mmail_mod $curr_mode
  listbox_restore $items $active
}


# Import a voice-message

proc do_import_voc {} {
  global do_FromAu_command do_FromWav_command do_FromVoc_command
  global do_FromLin_command do_FromBasic_command do_FromRmd_command
  global uservoc_dir startdir curr_mode curr_dir newname text

  if ![file writable $curr_dir] \
     {log_entry "Can't write to $curr_dir" ; return}
  set startdir $uservoc_dir
  set filename [select_file $uservoc_dir "Chose soundfile to import!"]
  set uservoc_dir $startdir
  if {"$filename" == ""} return
#.determine type of soundfile
  set ext [file extension $filename]
  if {"$ext" == ".au"} {set sound_type Au} \
  elseif {"$ext" == ".wav"} {set sound_type Wav} \
  elseif {"$ext" == ".voc"} {set sound_type Voc} \
  elseif {"$ext" == ".lin"} {set sound_type Lin} \
  elseif {"$ext" == ".bas"} {set sound_type Basic} \
  elseif {"$ext" == ".rmd"} {set sound_type Rmd} \
  else {
    log_entry "Can't determine type of file $filename"
    return
  }
  log_entry "Enter message name"
  set newname [file rootname [file tail $filename]]
  if {![interact entry Import newname] && "$newname" != ""} {
    set fullname $curr_dir/$newname
#...Check if there is already a file with this name
    if [file exists "$fullname"] {
      set text "Overwrite existing file $newname ?"
      set do_it ![interact label OK text]
    } else {set do_it 1}
    if $do_it {
      log_entry "Converting $filename to $fullname"
      set cmd "do_From${sound_type}_command"
      eval set cmd "$$cmd"
      log_entry "$cmd"
      if [catch "eval exec {cat $filename | $cmd > $fullname}" output] {
        log_entry "$output"
      }
    }
  }
  do_mmail_mod $curr_mode
}

# ------------------------------------------------------------------------------
# Index-Commands
# ------------------------------------------------------------------------------

# Activate or deactivate marked voice-messages in the Index-file

proc do_activate {} {
  global file_list hint_list index_db index_file plmwin

  set items [listbox_selection]
  if {$items == ""} {return 0}
  if ![file writable $index_file] \
     {log_entry "Activate: can't write to $index_file" ; return 0}
  set active [listbox_active]
  foreach line $items {
    set n [lindex $file_list $line]

    if {[lindex $hint_list $line] == "*"} {
      log_entry "$n: Deactivating file"
      index_delete "$n"
    } else {
      log_entry "$n: Activating file"
      lappend index_db "$n"
    }
  }
  write_Index
  $plmwin.button.update invoke
  listbox_restore $items $active
}


# Delete one Index-entry

proc index_delete {entry} {
  global index_db

  set i [lsearch -exact "$index_db" "$entry"]
  set index_db [del_listitem $index_db $i]
}


# Read the Index-file

proc read_Index {} {
  global curr_dir index_file index_db text

  create_file $index_file
  set index_db {}
  set error 0
  if {![catch {open $index_file r} file_id]} {
    while {![eof $file_id]} {
      gets $file_id line
      if {"$line" != "" && [lsearch -exact "$index_db" "$line"] < 0} {
        if {[file exists $curr_dir/$line] && "$line" != "beep" && "$line" != "Index"} {
          lappend index_db "$line"
        } else {set error 1}
      }
    }
    catch {close $file_id}
  }
  if $error {
    write_Index
    log_entry "WARNING: Corrected errors in Index"
  }
  if {"$index_db" == ""} {log_entry "WARNING: No active greeting message"}
}


# Write the Index-file

proc write_Index {} {
  global index_file index_db

  if {[catch {open $index_file w} file_id]} {
    log_entry "Can't write to $index_file"
  } else {
    foreach entry "$index_db" {
      if {"$entry" != ""} {puts $file_id "$entry"}
    }
    catch {close $file_id}
  }
}

